/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/
import React, { useEffect, useState } from 'react';
//Material ui componets
import { withStyles } from '@material-ui/core/styles';
import TableBody from '@material-ui/core/TableBody';
import Grid from "@material-ui/core/Grid";
import Paper from "@material-ui/core/Paper";
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import CheckRoundedIcon from "@material-ui/icons/CheckRounded";
import BlockRoundedIcon from "@material-ui/icons/BlockRounded";
import IconButton from '@material-ui/core/IconButton';
import VisibilityIcon from '@material-ui/icons/Visibility';
import { Button, Typography } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded'
//Local files
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
//Services
import { getRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import { Url } from '../../../Filters';
//routers
import { Link } from 'react-router-dom';
import styled from "styled-components"
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import TrendingUpRoundedIcon from "@material-ui/icons/TrendingUpRounded";

let currPage = 0; //var that controlls the current page that we are  
let transformListToAsc = false;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        '&:nth-of-type(odd)': {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const NoteVariables = () => {
    const WINDOW_WIDTH = window.innerWidth
    const AddOneLenght = [''];
    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete 
    const [items, setItems] = useState([]); //Necessary to consult the API, data
    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false)
    const [isUpdating, setIsUpdating] = useState(false)

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true)
        getRequest(
            api,
            (data, header) => {
                const arrData = [...data]
                if (arrData.length === 0) {
                    HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
                } else {
                    const arrItems = [...items]
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
                    const arrResult = arrItems.concat(arrData)
                    setItems(arrResult.concat(AddOneLenght))
                }
                setIsLoadingMoreItems(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsLoadingMoreItems(false)
            }
        )
    }

    const InvertList = async () => {
        transformListToAsc = !transformListToAsc
        currPage = 0
        if (transformListToAsc) {
            getRequest(
                Url('scores', '', `${currPage}`, 'ASC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(AddOneLenght))
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        } else {
            getRequest(
                Url('scores', '', `${currPage}`, 'DESC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(AddOneLenght))
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        }
    }

    const UpdateHandler = async (api) => {
        setIsUpdating(true)
        getRequest(
            api,
            (data, header) => {
                HandleSnack('A lista de dados foi atualizada', true, 'success', '#228B22')
                const arrData = [...data]
                setItems(arrData.concat(AddOneLenght))
                setIsUpdating(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsUpdating(false)
            }
        )
    }

    useEffect(() => {
        getRequest(
            Url('scores', '', '0', 'DESC'),
            (data, header) => {
                setIsLoaded(true);
                setItems(data.concat(AddOneLenght));
            },
            (error) => {
                setError(true);
            }
        )
    }, []);


    if (error) {
        return <div>Error: {error.message}</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {

        //Words in the top part of the table
        const topTable = ['ID', 'NOME', 'CÓDIGO', 'PESO', 'ATIVO', 'SCORE TYPE', 'VISUALIZAR'];

        if (WINDOW_WIDTH <= 961) {
            return (
                <div>
                    <SnackBar
                        severity={snackInfo.icon}
                        text={snackInfo.message}
                        snackbarOpen={snackInfo.open}
                        color={snackInfo.color}
                        handleClose={() => setSnackInfo({
                            message: '',
                            icon: '',
                            open: false,
                            color: ''
                        })}
                    />
                    <Paper style={{ padding: '1em' }}>
                        <Grid container spacing={3} direction="row" alignItems="center">
                            <Grid item xs={6}>
                                <Typography variant="h4">
                                    Variáveis de nota
                                    </Typography>
                            </Grid>
                            <Grid
                                item
                                xs={6}
                            >
                                <Grid container justify="flex-end" spacing={3}>
                                    <Grid item>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            disabled={isUpdating}
                                            onClick={() => {
                                                currPage = 0
                                                transformListToAsc = false
                                                UpdateHandler(Url('scores', '', `${currPage}`, 'DESC'))
                                            }}
                                            startIcon={<UpdateRoundedIcon />}
                                        >
                                            {
                                                isUpdating ? <CircularProgress size={24} /> : 'Atualizar'
                                            }
                                        </Button>
                                    </Grid>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Paper>

                    <div style={{ height: '2em' }}></div>

                    {items.map((row, index) =>
                        index === items.length - 1 ? (
                            <StyledDivButton>
                                <Button
                                    key={index}
                                    color="primary"
                                    variant="text"
                                    // disabled={isLoadingMoreItems}
                                    startIcon={<AddRoundedIcon />}
                                    disabled={isLoadingMoreItems}
                                    onClick={() => {
                                        currPage++
                                        if (transformListToAsc) {
                                            LoadMoreItens(Url('scores', '', `${currPage}`, 'ASC'))
                                        } else {
                                            LoadMoreItens(Url('scores', '', `${currPage}`, 'DESC'))
                                        }
                                    }}
                                >
                                    {isLoadingMoreItems ? (
                                        <CircularProgress size={24} />
                                    ) : (
                                            "Carregar mais itens"
                                        )}
                                </Button>
                            </StyledDivButton>
                        ) : (
                                <>
                                    <MobileList
                                        key={index}
                                        title={row.name}
                                        subtitle={row.id}
                                        backColor={"#e81f4f"}
                                        avatar={<TrendingUpRoundedIcon />}
                                        href={`/admin/noteVar/${row.id}`}
                                        reset={() => {
                                            currPage = 0;
                                        }}
                                        data={
                                            [
                                                {
                                                    title: "Código",
                                                    subtitle: row.code

                                                },
                                                {
                                                    title: "Score Type",
                                                    subtitle: row.score_type

                                                },
                                                {
                                                    title: "Ativo",
                                                    subtitle: row.active ? <CheckRoundedIcon style={{ fill: '#3CB371' }} /> : <BlockRoundedIcon style={{ fill: '#FA8072' }} />
                                                },
                                            ]
                                        }
                                    />
                                    <div style={{ height: "0.5em" }} />
                                </>
                            )
                    )}
                </div>
            )
        }
        else {
            return (
                <div>
                    <SnackBar
                        severity={snackInfo.icon}
                        text={snackInfo.message}
                        snackbarOpen={snackInfo.open}
                        color={snackInfo.color}
                        handleClose={() => setSnackInfo({
                            message: '',
                            icon: '',
                            open: false,
                            color: ''
                        })}
                    />
                    <Paper style={{ padding: '1em' }}>
                        <Grid container spacing={3} direction="row" alignItems="center">
                            <Grid item xs={6}>
                                <Typography variant="h4">
                                    Variáveis de nota
                                    </Typography>
                            </Grid>
                            <Grid
                                item
                                xs={6}
                            >
                                <Grid container justify="flex-end" spacing={3}>
                                    <Grid item>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            disabled={isUpdating}
                                            onClick={() => {
                                                currPage = 0
                                                transformListToAsc = false
                                                UpdateHandler(Url('scores', '', `${currPage}`, 'DESC'))
                                            }}
                                            startIcon={<UpdateRoundedIcon />}
                                        >
                                            {
                                                isUpdating ? <CircularProgress size={24}/> : 'Atualizar'
                                            }
                                        </Button>
                                    </Grid>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Paper>

                    <div style={{ height: '2em' }}></div>

                    <TableData
                        top={topTable}
                        onIconPressed={InvertList}
                    >
                        <TableBody>
                            {items.map((row, index) => (
                                index === items.length - 1 ?
                                    <StyledTableRow key={index}>
                                        {/* Button to load more data */}
                                        <StyledTableCell>
                                            <Button
                                                color='primary'
                                                variant='text'
                                                disabled={isLoadingMoreItems}
                                                startIcon={<AddRoundedIcon />}
                                                onClick={() => {
                                                    currPage++
                                                    if (transformListToAsc) {
                                                        LoadMoreItens(Url('scores', '', `${currPage}`, 'ASC'))
                                                    } else {
                                                        LoadMoreItens(Url('scores', '', `${currPage}`, 'DESC'))
                                                    }
                                                }}
                                            >
                                                {
                                                    isLoadingMoreItems ? <CircularProgress /> : 'Carregar mais itens'
                                                }
                                            </Button>
                                        </StyledTableCell>
                                    </StyledTableRow>

                                    :

                                    <StyledTableRow key={index}>
                                        <StyledTableCell component="th" scope="row">{row.id}</StyledTableCell>
                                        <StyledTableCell align="right">{row.name}</StyledTableCell>
                                        <StyledTableCell align="right">{row.code}</StyledTableCell>
                                        <StyledTableCell align="right">{row.weight}</StyledTableCell>
                                        <StyledTableCell align="right">
                                            {
                                                row.active ? <CheckRoundedIcon style={{ fill: '#3CB371' }} /> : <BlockRoundedIcon style={{ fill: '#FA8072' }} />
                                            }
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {
                                                row['score_type'].map((item) => (
                                                    <Typography key={item} style={{ fontSize: 14 }}>
                                                        {item}
                                                    </Typography>
                                                ))
                                            }
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            <Link to={`/admin/noteVar/${row.id}`}>
                                                <IconButton onClick={() => { currPage = 0; transformListToAsc = false }}>
                                                    <VisibilityIcon style={{ fill: '#00bcd4' }} />
                                                </IconButton>
                                            </Link>
                                        </StyledTableCell>
                                    </StyledTableRow>
                            ))}
                        </TableBody>
                    </TableData>
                </div>
            )
        }
    }
}


export default NoteVariables;

const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
`
