/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useState } from 'react';
//Material ui componets
import { withStyles } from '@material-ui/core/styles';
import TableBody from '@material-ui/core/TableBody';
import Grid from "@material-ui/core/Grid";
import Paper from "@material-ui/core/Paper";
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import IconButton from '@material-ui/core/IconButton';
import VisibilityIcon from '@material-ui/icons/Visibility';
import { Button, Typography } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import AddRoundedIcon from '@material-ui/icons/AddRounded';
import UpdateRoundedIcon from '@material-ui/icons/UpdateRounded';
import DeleteRoundedIcon from '@material-ui/icons/DeleteRounded';
//Local files
import TableData from '../../../Components/Components/Table';
import SnackBar from '../../../../Components/SnackbarComponent';
import LoadingSpinner from '../../../../Components/LoadingSpinner';
//Services
import AlertDialog from "../../../Components/Components/AlertDialog";
import { Url } from '../../../Filters';
import { DeleteFilter } from '../../../Filters';
import { getRequest, deleteRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//routers
import { Link } from 'react-router-dom';
import StarRoundedIcon from "@material-ui/icons/StarRounded";
import styled from "styled-components"
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"

let currPage = 0; //var that controlls the current page that we are  
let transformListToAsc = false;

const StyledTableCell = withStyles((theme) => ({
    head: {
        backgroundColor: theme.palette.common.black,
        color: theme.palette.common.white,
    },
    body: {
        fontSize: 14,
    },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
    root: {
        '&:nth-of-type(odd)': {
            backgroundColor: theme.palette.action.hover,
        },
    },
}))(TableRow);

const Ratings = () => {
    const WINDOW_WIDTH = window.innerWidth
    const AddOneLenght = [''];
    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete 
    const [items, setItems] = useState([]); //Necessary to consult the API, data
    const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false)
    const [isUpdating, setIsUpdating] = useState(false)

    const [openAlertDialog, setOpenAlertDialog] = useState(false); //controlls the state od alert dialog 
    const [deleteItem, setDeleteItem] = useState({}); //Delete Item
    const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const LoadMoreItens = async (api) => {
        setIsLoadingMoreItems(true)
        getRequest(
            api,
            (data, header) => {
                const arrData = [...data]
                if (arrData.length === 0) {
                    HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
                } else {
                    const arrItems = [...items]
                    arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
                    const arrResult = arrItems.concat(arrData)
                    setItems(arrResult.concat(AddOneLenght))
                }
                setIsLoadingMoreItems(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsLoadingMoreItems(false)
            }
        )
    }

    const InvertList = async () => {
        transformListToAsc = !transformListToAsc
        currPage = 0
        if (transformListToAsc) {
            getRequest(
                Url('ratings', '', `${currPage}`, 'ASC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(AddOneLenght))
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        } else {
            getRequest(
                Url('ratings', '', `${currPage}`, 'DESC'),
                (data, header) => {
                    const arrData = [...data]
                    setItems(arrData.concat(AddOneLenght))
                },
                (error) => {
                    HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                }
            )
        }
    }

    const UpdateHandler = async (api) => {
        setIsUpdating(true)
        getRequest(
            api,
            (data, header) => {
                HandleSnack('A lista de dados foi atualizada', true, 'success', '#228B22')
                const arrData = [...data]
                setItems(arrData.concat(AddOneLenght))
                setIsUpdating(false)
            },
            (error) => {
                HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
                setIsUpdating(false)
            }
        )
    }

    //handle Delete
    async function DeleteHandler() {
        const id = deleteItem.id;
        HandleStateAlertDialog(null);
        deleteRequest(
            DeleteFilter("ratings", id),
            (data) => {
                if (data.errors)
                    HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                else {
                    HandleSnack(
                        "O rating foi deletada com sucesso",
                        true,
                        "success",
                        "#228B22"
                    );
                    currPage = 0;
                    transformListToAsc = false
                    UpdateHandler(Url("ratings", "", `${currPage}`, "DESC"));
                    HandleStateCircularProgress(null);
                }
            },
            (error) => {
                HandleSnack("Ocorreu algum erro", true, "warning", "#FA8072");
                HandleStateCircularProgress(null);
            }
        )
    }

    const HandleStateCircularProgress = (i) => {
        setIsLoadingToDelete(i);
    };

    const HandleStateAlertDialog = (i) => {
        const obj = { ...items[i] };
        setDeleteItem(obj);
        setOpenAlertDialog(!openAlertDialog);
    };


    useEffect(() => {
        getRequest(
            Url('ratings', '', currPage, 'DESC'),
            (data, header) => {
                setIsLoaded(true);
                setItems(data.concat(AddOneLenght));
            },
            (error) => {
                setError(true)
            }
        )
    }, []);


    if (error) {
        return <div>Error: {error.message}</div>;
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {

        //Words in the top part of the table
        const topTable = ['ID', 'NOME', 'DESCRIÇÃO', 'VISUALIZAR', 'DELETAR'];
        if (WINDOW_WIDTH <= 760) {
            return (
                <div>
                    <SnackBar
                        severity={snackInfo.icon}
                        text={snackInfo.message}
                        snackbarOpen={snackInfo.open}
                        color={snackInfo.color}
                        handleClose={() => setSnackInfo({
                            message: '',
                            icon: '',
                            open: false,
                            color: ''
                        })}
                    />
                    <Paper style={{ padding: '1em' }}>
                        <Grid container spacing={3} direction="row" alignItems="center">
                            <Grid item xs={6}>
                                <Typography variant="h4">
                                    Lista de ratings
                                </Typography>
                            </Grid>
                            <Grid
                                item
                                xs={6}
                            >
                                <Grid container justify="flex-end" spacing={3}>
                                    <Grid item>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            disabled={isUpdating}
                                            onClick={() => {
                                                currPage = 0
                                                transformListToAsc = false
                                                UpdateHandler(Url('ratings', '', `${currPage}`, 'DESC'))
                                            }}
                                            startIcon={<UpdateRoundedIcon />}
                                        >
                                            {
                                                isUpdating ? <CircularProgress size={24} /> : 'Atualizar'
                                            }
                                        </Button>
                                    </Grid>
                                    <Grid item>
                                        <Link style={{ textDecoration: 'none' }} to={'/admin/CreateRating'}>
                                            <Button
                                                variant="contained"
                                                color="secondary"
                                                startIcon={<AddRoundedIcon style={{ fill: "white" }} />}
                                                onClick={() => {
                                                    currPage = 0
                                                    transformListToAsc = false
                                                }}
                                            >
                                                Novo
                                            </Button>
                                        </Link>
                                    </Grid>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Paper>

                    <div style={{ height: '2em' }}></div>

                    {items.map((row, index) =>
                        index === items.length - 1 ? (
                            <StyledDivButton>
                                <Button
                                    key={index}
                                    color="primary"
                                    variant="text"
                                    // disabled={isLoadingMoreItems}
                                    startIcon={<AddRoundedIcon />}
                                    disabled={isLoadingMoreItems}
                                    onClick={() => {
                                        currPage++
                                        if (transformListToAsc) {
                                            LoadMoreItens(Url('ratings', '', `${currPage}`, 'ASC'))
                                        } else {
                                            LoadMoreItens(Url('ratings', '', `${currPage}`, 'DESC'))
                                        }
                                    }}
                                >
                                    {isLoadingMoreItems ? (
                                        <CircularProgress size={24} />
                                    ) : (
                                            "Carregar mais itens"
                                        )}
                                </Button>
                            </StyledDivButton>
                        ) : (
                                <>
                                    <MobileList
                                        key={index}
                                        title={row.name}
                                        subtitle={row.id}
                                        backColor={"#00bcd4"}
                                        avatar={<StarRoundedIcon />}
                                        href={`/admin/Rating/${row.id}`}
                                        reset={() => {
                                            currPage = 0;
                                        }}
                                        data={
                                            [
                                                {
                                                    title: "Descrição",
                                                    subtitle: row.description
                                                }
                                            ]
                                        }
                                    />
                                    <div style={{ height: "0.5em" }} />
                                </>
                            )
                    )}
                </div>
            )
        }
        else {
            return (
                <div>
                    <SnackBar
                        severity={snackInfo.icon}
                        text={snackInfo.message}
                        snackbarOpen={snackInfo.open}
                        color={snackInfo.color}
                        handleClose={() => setSnackInfo({
                            message: '',
                            icon: '',
                            open: false,
                            color: ''
                        })}
                    />
                    <Paper style={{ padding: '1em' }}>
                        <Grid container spacing={3} direction="row" alignItems="center">
                            <Grid item xs={6}>
                                <Typography variant="h4">
                                    Lista de ratings
                                </Typography>
                            </Grid>
                            <Grid
                                item
                                xs={6}
                            >
                                <Grid container justify="flex-end" spacing={3}>
                                    <Grid item>
                                        <Button
                                            variant="contained"
                                            color="secondary"
                                            disabled={isUpdating}
                                            onClick={() => {
                                                currPage = 0
                                                transformListToAsc = false
                                                UpdateHandler(Url('ratings', '', `${currPage}`, 'DESC'))
                                            }}
                                            startIcon={<UpdateRoundedIcon />}
                                        >
                                            {
                                                isUpdating ? <CircularProgress size={24} /> : 'Atualizar'
                                            }
                                        </Button>
                                    </Grid>
                                    <Grid item>
                                        <Link style={{ textDecoration: 'none' }} to={'/admin/CreateRating'}>
                                            <Button
                                                variant="contained"
                                                color="secondary"
                                                startIcon={<AddRoundedIcon style={{ fill: "white" }} />}
                                                onClick={() => {
                                                    currPage = 0
                                                    transformListToAsc = false
                                                }}
                                            >
                                                Novo
                                            </Button>
                                        </Link>
                                    </Grid>
                                </Grid>
                            </Grid>
                        </Grid>
                    </Paper>

                    <div style={{ height: '2em' }}></div>

                    <TableData
                        top={topTable}
                        onIconPressed={InvertList}
                    >
                        <TableBody>
                            {items.map((row, index) => (
                                index === items.length - 1 ?
                                    <StyledTableRow key={index}>
                                        {/* Button to load more data */}
                                        <StyledTableCell>
                                            <Button
                                                color='primary'
                                                variant='text'
                                                disabled={isLoadingMoreItems}
                                                startIcon={<AddRoundedIcon />}
                                                onClick={() => {
                                                    currPage++
                                                    if (transformListToAsc) {
                                                        LoadMoreItens(Url('ratings', '', `${currPage}`, 'ASC'))
                                                    } else {
                                                        LoadMoreItens(Url('ratings', '', `${currPage}`, 'DESC'))
                                                    }
                                                }}
                                            >
                                                {
                                                    isLoadingMoreItems ? <CircularProgress size={24} /> : 'Carregar mais itens'
                                                }
                                            </Button>
                                        </StyledTableCell>
                                    </StyledTableRow>

                                    :

                                    <StyledTableRow key={index}>
                                        <StyledTableCell component="th" scope="row">{row.id}</StyledTableCell>
                                        <StyledTableCell align="right">{row.name}</StyledTableCell>
                                        <StyledTableCell align="right">{row.description}</StyledTableCell>
                                        <StyledTableCell align="right">
                                            <Link to={`/admin/Rating/${row.id}`}>
                                                <IconButton onClick={() => { currPage = 0 }}>
                                                    <VisibilityIcon style={{ fill: '#00bcd4' }} />
                                                </IconButton>
                                            </Link>
                                        </StyledTableCell>
                                        <StyledTableCell align="right">
                                            {isLoadingToDelete === index ? (
                                                <CircularProgress size={24} color="primary" />
                                            ) : (
                                                    <IconButton
                                                        onClick={() => {
                                                            HandleStateAlertDialog(index);
                                                            HandleStateCircularProgress(index);
                                                        }}
                                                    >
                                                        <DeleteRoundedIcon style={{ fill: "#FF0000" }} />
                                                    </IconButton>
                                                )}
                                        </StyledTableCell>
                                    </StyledTableRow>
                            ))}
                        </TableBody>
                    </TableData>

                    {/* This alert will be displayed if the user click to delete an institution */}
                    <AlertDialog
                        open={openAlertDialog}
                        OnDelete={DeleteHandler}
                        deleteItem={deleteItem}
                        HandleClose={() => {
                            setOpenAlertDialog(false);
                            HandleStateCircularProgress(null);
                        }}
                    />
                </div>
            )
        }
    }
}


export default Ratings;

const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
`

